/*
 * 版权所有 (C) 2015 知启蒙(ZHIQIM) 保留所有权利。[遇见知启蒙，邂逅框架梦]
 * 
 * https://zhiqim.org/project/zhiqim_framework/zhiqim_kernel.htm
 *
 * Zhiqim Kernel is licensed under Mulan PSL v2.
 * You can use this software according to the terms and conditions of the Mulan PSL v2.
 * You may obtain a copy of Mulan PSL v2 at:
 *          http://license.coscl.org.cn/MulanPSL2
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PSL v2 for more details.
 */
package org.zhiqim.kernel.schedule;

import org.zhiqim.kernel.annotation.AnAlias;
import org.zhiqim.kernel.util.DateTimes;

/**
 * 每(0-2,3-5,6-8,9-11,12-14,15-17,18-20,21-23)8时段小时任务，设置分、秒后在每天在指定点运行8次，每3小时运行一次
 * 时间格式：yyyyMMddHH[0-7]，0-7表示3小时取商数
 *  
 * @version v1.0.0 @author zouzhigang 2014-2-27 新建与整理
 */
@AnAlias("Hour3")
public final class Hour3 extends TaskThreader
{
    private int taskHour            = 0;            //指定处理的小时
    private int taskMinute          = 0;            //指定处理的分钟
    private int taskSecond          = 0;            //指定处理的秒
    
    /** 空构造函数,用于newInstance后再设置参数 */
    public Hour3(){}
    
    /**
     * 设置定时安排时间点，指定每日的时该任务运行
     * 
     * @param task      任务
     * @param hour      指定0-2小时
     * @param minute    指定分钟
     * @param second    指定秒
     */
    public Hour3(Task task, int hour, int minute, int second)
    {
        schedule(task, new int[]{hour, minute, second});
    }
    
    /**
     * 任务名称
     * 
     * @return          线程名称
     */
    public String getName()
    {
        return "Hour3";
    }
    
    /**
     * 任务时间安排
     * 
     * @return          时间安排
     */
    public String getTime()
    {
        return new StringBuilder().append(taskHour).append("/").append(taskMinute).append("/").append(taskSecond).toString();
    }
    
    /**
     * 时钟安排
     * 
     * @param task      任务
     * @param time      时间数组格式
     * @return          返回本对象
     */
    protected TaskThreader schedule(Task task, int[] time)
    {
        if (task == null || time == null || time.length != 3)
            throw new RuntimeException("Hour3传入的参数为空或数组长度不对");
        
        if (time[0] < 0 || time[0] > 2 || time[1] < 0 || time[1] > 59 || time[2] < 0 || time[2] > 59)
            throw new RuntimeException("Hour3传时的时间格式不正确");
        
        super.task = task;
        this.taskHour = time[0];
        this.taskMinute = time[1];
        this.taskSecond = time[2];
        return this;
    }
    
    @Override
    protected long calcLastTime(int curYearMonth, int curMonthMaxDay, int curDay, int curHour, int curMinute, int curSecond, int curWeek)
    {
        return curYearMonth * 1000L + curDay * 10 + curHour / 3;//取3小时的商
    }
    
    @Override
    protected long calcLastTimeNextTime()
    {
        long yearMonthDay = lastTime / 10;
        long curHourMod = lastTime % 10;
        
        if (curHourMod < 7)
            curHourMod += 1;
        else
        {
            curHourMod = 0;
            yearMonthDay = DateTimes.getNextDateInt((int)yearMonthDay);
        }
        
        return yearMonthDay * 10 + curHourMod;
    }
    
    @Override
    protected final boolean isArrive(int curYearMonth, int curMonthMaxDay, int curDay, int curHour, int curMinute, int curSecond, int curWeek)
    {
        int curHourMod = curHour % 3;//取3小时的余
        
        //下3小时，小时数未达
        if (curHourMod < taskHour)
            return false;
        
        //下3小时，小时数已过，一般由于不及时导致，表示到达
        if (curHourMod > taskHour)
            return true;
        
        //下3小时，小时相等，比较分钟，分钟未到达
        if (curMinute < taskMinute)
            return false;
        
        //下3小时，小时相等，比较分钟，分钟已达
        if (curMinute > taskMinute)
            return true;
        
        //下3小时，小时分钟相等，比较秒数
        return curSecond >= taskSecond;
    }

    public int getTaskHour()
    {
        return taskHour;
    }

    public int getTaskMinute()
    {
        return taskMinute;
    }

    public int getTaskSecond()
    {
        return taskSecond;
    }
}
